"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const apputils_1 = require("@jupyterlab/apputils");
const coreutils_1 = require("@jupyterlab/coreutils");
const services_1 = require("@jupyterlab/services");
const ui_components_1 = require("@jupyterlab/ui-components");
const algorithm_1 = require("@lumino/algorithm");
const coreutils_2 = require("@lumino/coreutils");
const domutils_1 = require("@lumino/domutils");
const dragdrop_1 = require("@lumino/dragdrop");
const polling_1 = require("@lumino/polling");
const signaling_1 = require("@lumino/signaling");
const widgets_1 = require("@lumino/widgets");
const scaling_1 = require("./scaling");
const React = require("react");
const ReactDOM = require("react-dom");
/**
 * A refresh interval (in ms) for polling the backend cluster manager.
 */
const REFRESH_INTERVAL = 5000;
/**
 * The threshold in pixels to start a drag event.
 */
const DRAG_THRESHOLD = 5;
/**
 * The mimetype used for Jupyter cell data.
 */
const JUPYTER_CELL_MIME = 'application/vnd.jupyter.cells';
/**
 * A widget for hosting Dask cluster management.
 */
class DaskClusterManager extends widgets_1.Widget {
    /**
     * Create a new Dask cluster manager.
     */
    constructor(options) {
        super();
        this._dragData = null;
        this._clusters = [];
        this._activeClusterChanged = new signaling_1.Signal(this);
        this._isDisposed = false;
        this._serverErrorShown = false;
        this._isReady = true;
        this.addClass('dask-DaskClusterManager');
        this._serverSettings = services_1.ServerConnection.makeSettings();
        this._injectClientCodeForCluster = options.injectClientCodeForCluster;
        this._getClientCodeForCluster = options.getClientCodeForCluster;
        this._registry = options.registry;
        this._launchClusterId = options.launchClusterId;
        // A function to set the active cluster.
        this._setActiveById = (id) => {
            const cluster = this._clusters.find(c => c.id === id);
            if (!cluster) {
                return;
            }
            const proxyUrl = coreutils_1.URLExt.join(this._serverSettings.baseUrl, 'proxy');
            const proxyPrefix = new URL(proxyUrl).pathname;
            if (cluster.dashboard_link.indexOf(proxyPrefix) !== -1) {
                // If the dashboard link is already proxied using
                // jupyter_server_proxy, don't proxy again. This
                // can happen if the user has overridden the dashboard
                // URL to the jupyter_server_proxy URL manually.
                options.setDashboardUrl(cluster.dashboard_link);
            }
            else {
                // Otherwise, use the internal proxy URL.
                options.setDashboardUrl(`dask/dashboard/${cluster.id}`);
            }
            const old = this._activeCluster;
            if (old && old.id === cluster.id) {
                return;
            }
            this._activeCluster = cluster;
            this._activeClusterChanged.emit({
                name: 'cluster',
                oldValue: old,
                newValue: cluster
            });
            this.update();
        };
        const layout = (this.layout = new widgets_1.PanelLayout());
        this._clusterListing = new widgets_1.Widget();
        this._clusterListing.addClass('dask-ClusterListing');
        // Create the toolbar.
        const toolbar = new apputils_1.Toolbar();
        // Make a label widget for the toolbar.
        const toolbarLabel = new widgets_1.Widget();
        toolbarLabel.node.textContent = 'CLUSTERS';
        toolbarLabel.addClass('dask-DaskClusterManager-label');
        toolbar.addItem('label', toolbarLabel);
        // Make a refresh button for the toolbar.
        toolbar.addItem('refresh', new apputils_1.ToolbarButton({
            icon: ui_components_1.refreshIcon,
            onClick: () => {
                this._updateClusterList();
            },
            tooltip: 'Refresh Cluster List'
        }));
        // Make a new cluster button for the toolbar.
        toolbar.addItem(this._launchClusterId, new apputils_1.CommandToolbarButton({
            commands: this._registry,
            id: this._launchClusterId
        }));
        layout.addWidget(toolbar);
        layout.addWidget(this._clusterListing);
        // Do an initial refresh of the cluster list.
        this._updateClusterList();
        // Also refresh periodically.
        this._poll = new polling_1.Poll({
            factory: () => __awaiter(this, void 0, void 0, function* () {
                yield this._updateClusterList();
            }),
            frequency: { interval: REFRESH_INTERVAL, backoff: true, max: 60 * 1000 },
            standby: 'when-hidden'
        });
    }
    /**
     * The currently selected cluster, or undefined if there is none.
     */
    get activeCluster() {
        return this._activeCluster;
    }
    /**
     * Set an active cluster by id.
     */
    setActiveCluster(id) {
        this._setActiveById(id);
    }
    /**
     * A signal that is emitted when an active cluster changes.
     */
    get activeClusterChanged() {
        return this._activeClusterChanged;
    }
    /**
     * Whether the cluster manager is ready to launch a cluster
     */
    get isReady() {
        return this._isReady;
    }
    /**
     * Get the current clusters known to the manager.
     */
    get clusters() {
        return this._clusters;
    }
    /**
     * Refresh the current list of clusters.
     */
    refresh() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this._updateClusterList();
        });
    }
    /**
     * Start a new cluster.
     */
    start() {
        return __awaiter(this, void 0, void 0, function* () {
            const cluster = yield this._launchCluster();
            return cluster;
        });
    }
    /**
     * Stop a cluster by ID.
     */
    stop(id) {
        return __awaiter(this, void 0, void 0, function* () {
            const cluster = this._clusters.find(c => c.id === id);
            if (!cluster) {
                throw Error(`Cannot find cluster ${id}`);
            }
            yield this._stopById(id);
        });
    }
    /**
     * Scale a cluster by ID.
     */
    scale(id) {
        return __awaiter(this, void 0, void 0, function* () {
            const cluster = this._clusters.find(c => c.id === id);
            if (!cluster) {
                throw Error(`Cannot find cluster ${id}`);
            }
            const newCluster = yield this._scaleById(id);
            return newCluster;
        });
    }
    /**
     * Dispose of the cluster manager.
     */
    dispose() {
        if (this._isDisposed) {
            return;
        }
        this._isDisposed = true;
        this._poll.dispose();
        super.dispose();
    }
    /**
     * Whether the dashboard has been disposed.
     */
    get isDisposed() {
        return this._isDisposed;
    }
    /**
     * Handle an update request.
     */
    onUpdateRequest(msg) {
        // Don't bother if the sidebar is not visible
        if (!this.isVisible) {
            return;
        }
        ReactDOM.render(React.createElement(ClusterListing, { clusters: this._clusters, activeClusterId: (this._activeCluster && this._activeCluster.id) || '', scaleById: (id) => {
                return this._scaleById(id);
            }, stopById: (id) => {
                return this._stopById(id);
            }, setActiveById: this._setActiveById, injectClientCodeForCluster: this._injectClientCodeForCluster }), this._clusterListing.node);
    }
    /**
     * Rerender after showing.
     */
    onAfterShow(msg) {
        this.update();
    }
    /**
     * Handle `after-attach` messages for the widget.
     */
    onAfterAttach(msg) {
        super.onAfterAttach(msg);
        let node = this._clusterListing.node;
        node.addEventListener('p-dragenter', this);
        node.addEventListener('p-dragleave', this);
        node.addEventListener('p-dragover', this);
        node.addEventListener('mousedown', this);
    }
    /**
     * Handle `before-detach` messages for the widget.
     */
    onBeforeDetach(msg) {
        let node = this._clusterListing.node;
        node.removeEventListener('p-dragenter', this);
        node.removeEventListener('p-dragleave', this);
        node.removeEventListener('p-dragover', this);
        node.removeEventListener('mousedown', this);
        document.removeEventListener('mouseup', this, true);
        document.removeEventListener('mousemove', this, true);
    }
    /**
     * Handle the DOM events for the directory listing.
     *
     * @param event - The DOM event sent to the widget.
     *
     * #### Notes
     * This method implements the DOM `EventListener` interface and is
     * called in response to events on the panel's DOM node. It should
     * not be called directly by user code.
     */
    handleEvent(event) {
        switch (event.type) {
            case 'mousedown':
                this._evtMouseDown(event);
                break;
            case 'mouseup':
                this._evtMouseUp(event);
                break;
            case 'mousemove':
                this._evtMouseMove(event);
                break;
            default:
                break;
        }
    }
    /**
     * Handle `mousedown` events for the widget.
     */
    _evtMouseDown(event) {
        const { button, shiftKey } = event;
        // We only handle main or secondary button actions.
        if (!(button === 0 || button === 2)) {
            return;
        }
        // Shift right-click gives the browser default behavior.
        if (shiftKey && button === 2) {
            return;
        }
        // Find the target cluster.
        const clusterIndex = this._findCluster(event);
        if (clusterIndex === -1) {
            return;
        }
        // Prepare for a drag start
        this._dragData = {
            pressX: event.clientX,
            pressY: event.clientY,
            index: clusterIndex
        };
        // Enter possible drag mode
        document.addEventListener('mouseup', this, true);
        document.addEventListener('mousemove', this, true);
        event.preventDefault();
    }
    /**
     * Handle the `'mouseup'` event on the document.
     */
    _evtMouseUp(event) {
        // Remove the event listeners we put on the document
        if (event.button !== 0 || !this._drag) {
            document.removeEventListener('mousemove', this, true);
            document.removeEventListener('mouseup', this, true);
        }
        event.preventDefault();
        event.stopPropagation();
    }
    /**
     * Handle the `'mousemove'` event for the widget.
     */
    _evtMouseMove(event) {
        let data = this._dragData;
        if (!data) {
            return;
        }
        // Check for a drag initialization.
        let dx = Math.abs(event.clientX - data.pressX);
        let dy = Math.abs(event.clientY - data.pressY);
        if (dx >= DRAG_THRESHOLD || dy >= DRAG_THRESHOLD) {
            event.preventDefault();
            event.stopPropagation();
            this._startDrag(data.index, event.clientX, event.clientY);
        }
    }
    /**
     * Start a drag event.
     */
    _startDrag(index, clientX, clientY) {
        // Create the drag image.
        const model = this._clusters[index];
        const listingItem = this._clusterListing.node.querySelector(`li.dask-ClusterListingItem[data-cluster-id="${model.id}"]`);
        const dragImage = Private.createDragImage(listingItem);
        // Set up the drag event.
        this._drag = new dragdrop_1.Drag({
            mimeData: new coreutils_2.MimeData(),
            dragImage,
            supportedActions: 'copy',
            proposedAction: 'copy',
            source: this
        });
        // Add mimeData for plain text so that normal editors can
        // receive the data.
        const textData = this._getClientCodeForCluster(model);
        this._drag.mimeData.setData('text/plain', textData);
        // Add cell data for notebook drops.
        const cellData = [
            {
                cell_type: 'code',
                source: textData,
                outputs: [],
                execution_count: null,
                metadata: {}
            }
        ];
        this._drag.mimeData.setData(JUPYTER_CELL_MIME, cellData);
        // Remove mousemove and mouseup listeners and start the drag.
        document.removeEventListener('mousemove', this, true);
        document.removeEventListener('mouseup', this, true);
        this._drag.start(clientX, clientY).then(action => {
            if (this.isDisposed) {
                return;
            }
            this._drag = null;
            this._dragData = null;
        });
    }
    /**
     * Launch a new cluster on the server.
     */
    _launchCluster() {
        return __awaiter(this, void 0, void 0, function* () {
            this._isReady = false;
            this._registry.notifyCommandChanged(this._launchClusterId);
            const response = yield services_1.ServerConnection.makeRequest(`${this._serverSettings.baseUrl}dask/clusters`, { method: 'PUT' }, this._serverSettings);
            if (response.status !== 200) {
                const err = yield response.json();
                void apputils_1.showErrorMessage('Cluster Start Error', err);
                this._isReady = true;
                this._registry.notifyCommandChanged(this._launchClusterId);
                throw err;
            }
            const model = (yield response.json());
            yield this._updateClusterList();
            this._isReady = true;
            this._registry.notifyCommandChanged(this._launchClusterId);
            return model;
        });
    }
    /**
     * Refresh the list of clusters on the server.
     */
    _updateClusterList() {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield services_1.ServerConnection.makeRequest(`${this._serverSettings.baseUrl}dask/clusters`, {}, this._serverSettings);
            if (response.status !== 200) {
                const msg = 'Failed to list clusters: might the server extension not be installed/enabled?';
                const err = new Error(msg);
                if (!this._serverErrorShown) {
                    apputils_1.showErrorMessage('Dask Server Error', err);
                    this._serverErrorShown = true;
                }
                throw err;
            }
            const data = (yield response.json());
            this._clusters = data;
            // Check to see if the active cluster still exits.
            // If it doesn't, or if there is no active cluster,
            // select the first one.
            const active = this._clusters.find(c => c.id === (this._activeCluster && this._activeCluster.id));
            if (!active) {
                const id = (this._clusters[0] && this._clusters[0].id) || '';
                this._setActiveById(id);
            }
            this.update();
        });
    }
    /**
     * Stop a cluster by its id.
     */
    _stopById(id) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield services_1.ServerConnection.makeRequest(`${this._serverSettings.baseUrl}dask/clusters/${id}`, { method: 'DELETE' }, this._serverSettings);
            if (response.status !== 204) {
                const err = yield response.json();
                void apputils_1.showErrorMessage('Failed to close cluster', err);
                throw err;
            }
            yield this._updateClusterList();
        });
    }
    /**
     * Scale a cluster by its id.
     */
    _scaleById(id) {
        return __awaiter(this, void 0, void 0, function* () {
            const cluster = this._clusters.find(c => c.id === id);
            if (!cluster) {
                throw Error(`Failed to find cluster ${id} to scale`);
            }
            const update = yield scaling_1.showScalingDialog(cluster);
            if (coreutils_2.JSONExt.deepEqual(update, cluster)) {
                // If the user canceled, or the model is identical don't try to update.
                return Promise.resolve(cluster);
            }
            const response = yield services_1.ServerConnection.makeRequest(`${this._serverSettings.baseUrl}dask/clusters/${id}`, {
                method: 'PATCH',
                body: JSON.stringify(update)
            }, this._serverSettings);
            if (response.status !== 200) {
                const err = yield response.json();
                void apputils_1.showErrorMessage('Failed to scale cluster', err);
                throw err;
            }
            const model = (yield response.json());
            yield this._updateClusterList();
            return model;
        });
    }
    _findCluster(event) {
        const nodes = Array.from(this.node.querySelectorAll('li.dask-ClusterListingItem'));
        return algorithm_1.ArrayExt.findFirstIndex(nodes, node => {
            return domutils_1.ElementExt.hitTest(node, event.clientX, event.clientY);
        });
    }
}
exports.DaskClusterManager = DaskClusterManager;
/**
 * A React component for a launcher button listing.
 */
function ClusterListing(props) {
    let listing = props.clusters.map(cluster => {
        return (React.createElement(ClusterListingItem, { isActive: cluster.id === props.activeClusterId, key: cluster.id, cluster: cluster, scale: () => props.scaleById(cluster.id), stop: () => props.stopById(cluster.id), setActive: () => props.setActiveById(cluster.id), injectClientCode: () => props.injectClientCodeForCluster(cluster) }));
    });
    // Return the JSX component.
    return (React.createElement("div", null,
        React.createElement("ul", { className: "dask-ClusterListing-list" }, listing)));
}
/**
 * A TSX functional component for rendering a single running cluster.
 */
function ClusterListingItem(props) {
    const { cluster, isActive, setActive, scale, stop, injectClientCode } = props;
    let itemClass = 'dask-ClusterListingItem';
    itemClass = isActive ? `${itemClass} jp-mod-active` : itemClass;
    let minimum = null;
    let maximum = null;
    if (cluster.adapt) {
        minimum = (React.createElement("div", { className: "dask-ClusterListingItem-stats" },
            "Minimum Workers: ",
            cluster.adapt.minimum));
        maximum = (React.createElement("div", { className: "dask-ClusterListingItem-stats" },
            "Maximum Workers: ",
            cluster.adapt.maximum));
    }
    return (React.createElement("li", { className: itemClass, "data-cluster-id": cluster.id, onClick: evt => {
            setActive();
            evt.stopPropagation();
        } },
        React.createElement("div", { className: "dask-ClusterListingItem-title" }, cluster.name),
        React.createElement("div", { className: "dask-ClusterListingItem-link", title: cluster.scheduler_address },
            "Scheduler Address: ",
            cluster.scheduler_address),
        React.createElement("div", { className: "dask-ClusterListingItem-link" },
            "Dashboard URL:",
            ' ',
            React.createElement("a", { target: "_blank", href: cluster.dashboard_link, title: cluster.dashboard_link }, cluster.dashboard_link)),
        React.createElement("div", { className: "dask-ClusterListingItem-stats" },
            "Number of Cores: ",
            cluster.cores),
        React.createElement("div", { className: "dask-ClusterListingItem-stats" },
            "Memory: ",
            cluster.memory),
        React.createElement("div", { className: "dask-ClusterListingItem-stats" },
            "Number of Workers: ",
            cluster.workers),
        minimum,
        maximum,
        React.createElement("div", { className: "dask-ClusterListingItem-button-panel" },
            React.createElement("button", { className: "dask-ClusterListingItem-button dask-ClusterListingItem-code dask-CodeIcon jp-mod-styled", onClick: evt => {
                    injectClientCode();
                    evt.stopPropagation();
                }, title: `Inject client code for ${cluster.name}` }),
            React.createElement("button", { className: "dask-ClusterListingItem-button dask-ClusterListingItem-scale jp-mod-styled", onClick: evt => {
                    scale();
                    evt.stopPropagation();
                }, title: `Rescale ${cluster.name}` }, "SCALE"),
            React.createElement("button", { className: "dask-ClusterListingItem-button dask-ClusterListingItem-stop jp-mod-styled", onClick: evt => {
                    stop();
                    evt.stopPropagation();
                }, title: `Shutdown ${cluster.name}` }, "SHUTDOWN"))));
}
/**
 * A namespace for module-private functionality.
 */
var Private;
(function (Private) {
    /**
     * Create a drag image for an HTML node.
     */
    function createDragImage(node) {
        const image = node.cloneNode(true);
        image.classList.add('dask-ClusterListingItem-drag');
        return image;
    }
    Private.createDragImage = createDragImage;
})(Private || (Private = {}));
