import { IFrame, MainAreaWidget } from '@jupyterlab/apputils';
import { ServerConnection } from '@jupyterlab/services';
import { JSONObject } from '@lumino/coreutils';
import { Message } from '@lumino/messaging';
import { ISignal } from '@lumino/signaling';
import { Widget } from '@lumino/widgets';
/**
 * A class for hosting a Dask dashboard in an iframe.
 */
export declare class DaskDashboard extends MainAreaWidget<IFrame> {
    /**
     * Construct a new dashboard widget.
     */
    constructor();
    /**
     * The current dashboard item for the widget.
     */
    get item(): IDashboardItem | null;
    set item(value: IDashboardItem | null);
    /**
     * The current dashboard URL for the widget.
     */
    get dashboardUrl(): string;
    set dashboardUrl(value: string);
    /**
     * Whether the dashboard is active. When inactive,
     * it will show a placeholder panel.
     */
    get active(): boolean;
    set active(value: boolean);
    /**
     * Handle an update request to the dashboard panel.
     */
    protected onUpdateRequest(): void;
    private _item;
    private _dashboardUrl;
    private _active;
    private _inactivePanel;
}
/**
 * A widget for hosting Dask dashboard launchers.
 */
export declare class DaskDashboardLauncher extends Widget {
    /**
     * Create a new Dask sidebar.
     */
    constructor(options: DaskDashboardLauncher.IOptions);
    private updateLinks;
    /**
     * The list of dashboard items which can be launched.
     */
    get items(): IDashboardItem[];
    /**
     * Get the URL input widget.
     */
    get input(): URLInput;
    /**
     * Handle an update request.
     */
    protected onUpdateRequest(msg: Message): void;
    /**
     * Rerender after showing.
     */
    protected onAfterShow(msg: Message): void;
    private _dashboard;
    private _input;
    private _launchItem;
    private _items;
    private _serverSettings;
}
/**
 * A widget for hosting a url input element.
 */
export declare class URLInput extends Widget {
    /**
     * Construct a new input element.
     */
    constructor(serverSettings: ServerConnection.ISettings, linkFinder?: () => Promise<string>);
    /**
     * The underlying input value.
     */
    get input(): HTMLInputElement;
    /**
     * The base url for the dask webserver.
     *
     * #### Notes
     * Setting this value will result in a urlChanged
     * signal being emitted, but it will happen asynchronously,
     * as it first checks to see whether the url is pointing
     * at a valid dask dashboard server.
     */
    get url(): string;
    set url(newValue: string);
    /**
     * Whether the current url is pointing to a valid dask dashboard.
     */
    get isValid(): boolean;
    /**
     * A signal emitted when the url changes.
     */
    get urlChanged(): ISignal<this, URLInput.IChangedArgs>;
    /**
     * Dispose of the resources held by the dashboard.
     */
    dispose(): void;
    /**
     * Whether the dashboard has been disposed.
     */
    get isDisposed(): boolean;
    /**
     * Handle the DOM events for the widget.
     *
     * @param event - The DOM event sent to the widget.
     *
     * #### Notes
     * This method implements the DOM `EventListener` interface and is
     * called in response to events on the main area widget's node. It should
     * not be called directly by user code.
     */
    handleEvent(event: KeyboardEvent): void;
    /**
     * Handle `after-attach` messages for the widget.
     */
    protected onAfterAttach(msg: Message): void;
    /**
     * Handle `before-detach` messages for the widget.
     */
    protected onBeforeDetach(msg: Message): void;
    /**
     * Periodically poll for valid url.
     */
    private _startUrlCheckTimer;
    private _urlChanged;
    private _url;
    private _isValid;
    private _input;
    private _poll;
    private _isDisposed;
    private _serverSettings;
}
/**
 * A namespace for URLInput statics.
 */
export declare namespace URLInput {
    /**
     * Changed args for the url.
     */
    interface IChangedArgs {
        /**
         * The old url.
         */
        oldValue: string;
        /**
         * The new url.
         */
        newValue: string;
        /**
         * Whether the URL is pointing at a valid dask webserver.
         */
        isValid: boolean;
    }
}
/**
 * A namespace for DaskDashboardLauncher statics.
 */
export declare namespace DaskDashboardLauncher {
    /**
     * Options for the constructor.
     */
    interface IOptions {
        /**
         * A function that attempts to find a link to
         * a dask bokeh server in the current application
         * context.
         */
        linkFinder?: () => Promise<string>;
        /**
         * A callback to launch a dashboard item.
         */
        launchItem: (item: IDashboardItem) => void;
        /**
         * A list of items for the launcher.
         */
        items?: IDashboardItem[];
    }
    const DEFAULT_ITEMS: {
        route: string;
        label: string;
    }[];
}
/**
 * Props for the dashboard listing component.
 */
export interface IDashboardListingProps {
    /**
     * A list of dashboard items to render.
     */
    items: IDashboardItem[];
    /**
     * A callback to launch a dashboard item.
     */
    launchItem: (item: IDashboardItem) => void;
    /**
     * Whether the items should be enabled.
     */
    isEnabled: boolean;
}
/**
 * An interface dashboard launcher item.
 */
export interface IDashboardItem extends JSONObject {
    /**
     * The route to add the the base url.
     */
    route: string;
    /**
     * The display label for the item.
     */
    label: string;
}
