"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const apputils_1 = require("@jupyterlab/apputils");
const coreutils_1 = require("@jupyterlab/coreutils");
const services_1 = require("@jupyterlab/services");
const ui_components_1 = require("@jupyterlab/ui-components");
const coreutils_2 = require("@lumino/coreutils");
const polling_1 = require("@lumino/polling");
const signaling_1 = require("@lumino/signaling");
const widgets_1 = require("@lumino/widgets");
const React = require("react");
const ReactDOM = require("react-dom");
/**
 * A class for hosting a Dask dashboard in an iframe.
 */
class DaskDashboard extends apputils_1.MainAreaWidget {
    /**
     * Construct a new dashboard widget.
     */
    constructor() {
        super({
            // Disable allow some iframe extensions to let server requests
            // and scripts to execute in the bokeh server context.
            // This is unsafe, but we presumably trust the code in the bokeh server.
            content: new apputils_1.IFrame({ sandbox: ['allow-scripts', 'allow-same-origin'] })
        });
        this._item = null;
        this._dashboardUrl = '';
        this._active = false;
        this._inactivePanel = Private.createInactivePanel();
        this.content.node.appendChild(this._inactivePanel);
        this.update();
    }
    /**
     * The current dashboard item for the widget.
     */
    get item() {
        return this._item;
    }
    set item(value) {
        if (coreutils_2.JSONExt.deepEqual(value, this._item)) {
            return;
        }
        this._item = value;
        this.update();
    }
    /**
     * The current dashboard URL for the widget.
     */
    get dashboardUrl() {
        return this._dashboardUrl;
    }
    set dashboardUrl(value) {
        if (value === this._dashboardUrl) {
            return;
        }
        this._dashboardUrl = Private.normalizeDashboardUrl(value);
        this.update();
    }
    /**
     * Whether the dashboard is active. When inactive,
     * it will show a placeholder panel.
     */
    get active() {
        return this._active;
    }
    set active(value) {
        if (value === this._active) {
            return;
        }
        this._active = value;
        this.update();
    }
    /**
     * Handle an update request to the dashboard panel.
     */
    onUpdateRequest() {
        // If there is nothing to show, empty the iframe URL and
        // show the inactive panel.
        if (!this.item || !this.dashboardUrl || !this.active) {
            this.content.url = '';
            this._inactivePanel.style.display = '';
            return;
        }
        // Make sure the inactive panel is hidden
        this._inactivePanel.style.display = 'none';
        this.content.url = coreutils_1.URLExt.join(this.dashboardUrl, this.item.route);
    }
}
exports.DaskDashboard = DaskDashboard;
/**
 * A widget for hosting Dask dashboard launchers.
 */
class DaskDashboardLauncher extends widgets_1.Widget {
    /**
     * Create a new Dask sidebar.
     */
    constructor(options) {
        super();
        let layout = (this.layout = new widgets_1.PanelLayout());
        this._dashboard = new widgets_1.Widget();
        this._serverSettings = services_1.ServerConnection.makeSettings();
        this._input = new URLInput(this._serverSettings, options.linkFinder);
        layout.addWidget(this._input);
        layout.addWidget(this._dashboard);
        this.addClass('dask-DaskDashboardLauncher');
        this._items = options.items || DaskDashboardLauncher.DEFAULT_ITEMS;
        this._launchItem = options.launchItem;
        this._input.urlChanged.connect(this.updateLinks, this);
    }
    updateLinks() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._input.isValid) {
                this.update();
                return;
            }
            const result = yield Private.getItems(this._input.url, this._serverSettings);
            this._items = result || DaskDashboardLauncher.DEFAULT_ITEMS;
            this.update();
        });
    }
    /**
     * The list of dashboard items which can be launched.
     */
    get items() {
        return this._items;
    }
    /**
     * Get the URL input widget.
     */
    get input() {
        return this._input;
    }
    /**
     * Handle an update request.
     */
    onUpdateRequest(msg) {
        // Don't bother if the sidebar is not visible
        if (!this.isVisible) {
            return;
        }
        ReactDOM.render(React.createElement(DashboardListing, { launchItem: this._launchItem, isEnabled: this.input.isValid, items: this._items }), this._dashboard.node);
    }
    /**
     * Rerender after showing.
     */
    onAfterShow(msg) {
        this.update();
    }
}
exports.DaskDashboardLauncher = DaskDashboardLauncher;
/**
 * A widget for hosting a url input element.
 */
class URLInput extends widgets_1.Widget {
    /**
     * Construct a new input element.
     */
    constructor(serverSettings, linkFinder) {
        super();
        this._urlChanged = new signaling_1.Signal(this);
        this._url = '';
        this._isValid = false;
        this.addClass('dask-URLInput');
        const layout = (this.layout = new widgets_1.PanelLayout());
        const wrapper = new widgets_1.Widget();
        wrapper.addClass('dask-URLInput-wrapper');
        this._input = document.createElement('input');
        this._input.placeholder = 'DASK DASHBOARD URL';
        wrapper.node.appendChild(this._input);
        layout.addWidget(wrapper);
        this._serverSettings = serverSettings;
        if (linkFinder) {
            const findButton = new apputils_1.ToolbarButton({
                icon: ui_components_1.searchIcon,
                onClick: () => __awaiter(this, void 0, void 0, function* () {
                    let link = yield linkFinder();
                    if (link) {
                        this.url = link;
                    }
                }),
                tooltip: 'Auto-detect dashboard URL'
            });
            layout.addWidget(findButton);
        }
        this._startUrlCheckTimer();
    }
    /**
     * The underlying input value.
     */
    get input() {
        return this._input;
    }
    /**
     * The base url for the dask webserver.
     *
     * #### Notes
     * Setting this value will result in a urlChanged
     * signal being emitted, but it will happen asynchronously,
     * as it first checks to see whether the url is pointing
     * at a valid dask dashboard server.
     */
    get url() {
        return this._url;
    }
    set url(newValue) {
        this._input.value = newValue;
        const oldValue = this._url;
        if (newValue === oldValue) {
            return;
        }
        Private.testDaskDashboard(newValue, this._serverSettings).then(result => {
            this._url = newValue;
            this._isValid = result;
            this._urlChanged.emit({ isValid: result, oldValue, newValue });
            this._input.blur();
            this.update();
            if (!result) {
                console.warn(`${newValue} does not appear to host a valid Dask dashboard`);
            }
        });
    }
    /**
     * Whether the current url is pointing to a valid dask dashboard.
     */
    get isValid() {
        return this._isValid;
    }
    /**
     * A signal emitted when the url changes.
     */
    get urlChanged() {
        return this._urlChanged;
    }
    /**
     * Dispose of the resources held by the dashboard.
     */
    dispose() {
        if (this._isDisposed) {
            return;
        }
        this._isDisposed = true;
        this._poll.dispose();
        super.dispose();
    }
    /**
     * Whether the dashboard has been disposed.
     */
    get isDisposed() {
        return this._isDisposed;
    }
    /**
     * Handle the DOM events for the widget.
     *
     * @param event - The DOM event sent to the widget.
     *
     * #### Notes
     * This method implements the DOM `EventListener` interface and is
     * called in response to events on the main area widget's node. It should
     * not be called directly by user code.
     */
    handleEvent(event) {
        switch (event.type) {
            case 'keydown':
                switch (event.keyCode) {
                    case 13: // Enter
                        event.stopPropagation();
                        event.preventDefault();
                        this.url = this._input.value;
                        break;
                    default:
                        break;
                }
                break;
            default:
                break;
        }
    }
    /**
     * Handle `after-attach` messages for the widget.
     */
    onAfterAttach(msg) {
        this._input.addEventListener('keydown', this, true);
    }
    /**
     * Handle `before-detach` messages for the widget.
     */
    onBeforeDetach(msg) {
        this._input.removeEventListener('keydown', this, true);
    }
    /**
     * Periodically poll for valid url.
     */
    _startUrlCheckTimer() {
        this._poll = new polling_1.Poll({
            factory: () => __awaiter(this, void 0, void 0, function* () {
                const url = this._url;
                // Don't bother checking if there is no url.
                if (!url) {
                    return;
                }
                const result = yield Private.testDaskDashboard(url, this._serverSettings);
                // No change.
                if (result === this._isValid) {
                    return;
                }
                // Show an error if the connection died.
                if (!result && this._isValid) {
                    console.warn(`The connection to dask dashboard ${url} has been lost`);
                }
                // Connection died or started
                if (result !== this._isValid) {
                    this._isValid = result;
                    this._urlChanged.emit({
                        oldValue: url,
                        newValue: url,
                        isValid: result
                    });
                }
            }),
            frequency: { interval: 4 * 1000, backoff: true, max: 60 * 1000 },
            standby: 'when-hidden'
        });
    }
}
exports.URLInput = URLInput;
/**
 * A namespace for DaskDashboardLauncher statics.
 */
(function (DaskDashboardLauncher) {
    DaskDashboardLauncher.DEFAULT_ITEMS = [
        { route: 'individual-task-stream', label: 'Task Stream' },
        { route: 'individual-progress', label: 'Progress' },
        { route: 'individual-graph', label: 'Graph' },
        { route: 'individual-workers', label: 'Workers' },
        { route: 'individual-nbytes', label: 'Memory (worker)' },
        { route: 'individual-memory-by-key', label: 'Memory (operation)' },
        { route: 'individual-nprocessing', label: 'Processing Tasks' },
        { route: 'individual-profile', label: 'Profile' },
        { route: 'individual-profile-server', label: 'Profile Server' },
        { route: 'individual-cpu', label: 'CPU (workers)' },
        { route: 'individual-bandwidth-workers', label: 'Bandwidth (workers)' },
        { route: 'individual-bandwidth-types', label: 'Bandwidth (type)' }
    ];
})(DaskDashboardLauncher = exports.DaskDashboardLauncher || (exports.DaskDashboardLauncher = {}));
/**
 * A React component for a launcher button listing.
 */
function DashboardListing(props) {
    let listing = props.items.map(item => {
        return (React.createElement("li", { className: "dask-DashboardListing-item", key: item.route },
            React.createElement("button", { className: "jp-mod-styled jp-mod-accept", value: item.label, disabled: !props.isEnabled, onClick: () => props.launchItem(item) }, item.label)));
    });
    // Return the JSX component.
    return (React.createElement("div", null,
        React.createElement("ul", { className: "dask-DashboardListing-list" }, listing)));
}
/**
 * A namespace for private functionality.
 */
var Private;
(function (Private) {
    /**
     * Optionally remove a `status` route from a dashboard url.
     */
    function normalizeDashboardUrl(url, baseUrl = '') {
        if (isLocal(url)) {
            if (!baseUrl) {
                baseUrl = coreutils_1.PageConfig.getBaseUrl();
            }
            // If the path-portion of the baseUrl has been included,
            // strip that off.
            const tmp = new URL(baseUrl);
            if (url.startsWith(tmp.pathname)) {
                url = url.slice(tmp.pathname.length);
            }
            // Fully qualify the local URL to remove any relative-path confusion.
            url = baseUrl + url;
        }
        // If 'status' has been included at the end, strip it.
        if (url.endsWith('status')) {
            url = url.slice(0, -'status'.length);
        }
        else if (url.endsWith('status/')) {
            url = url.slice(0, -'status/'.length);
        }
        return url;
    }
    Private.normalizeDashboardUrl = normalizeDashboardUrl;
    /**
     * Return the json result of /individual-plots.json
     */
    function getItems(url, settings) {
        return __awaiter(this, void 0, void 0, function* () {
            // We can't fetch items from a non-local URL due to CORS policies.
            if (!isLocal(url)) {
                return DaskDashboardLauncher.DEFAULT_ITEMS;
            }
            url = normalizeDashboardUrl(url, settings.baseUrl);
            const response = yield services_1.ServerConnection.makeRequest(coreutils_1.URLExt.join(url, 'individual-plots.json'), {}, settings);
            if (response.status === 200) {
                let result = yield response.json();
                const newItems = [];
                for (let key in result) {
                    let label = key.replace('Individual ', '');
                    let route = String(result[key]);
                    let item = { route: route, label: label, key: label };
                    newItems.push(item);
                }
                return newItems;
            }
            else {
                return [];
            }
        });
    }
    Private.getItems = getItems;
    /**
     * Test whether a given URL hosts a dask dashboard.
     */
    function testDaskDashboard(url, settings) {
        url = normalizeDashboardUrl(url, settings.baseUrl);
        // If this is a url that we are proxying under the notebook server,
        // it is easier to check for a valid dashboard.
        if (url.indexOf(settings.baseUrl) === 0) {
            return services_1.ServerConnection.makeRequest(coreutils_1.URLExt.join(url, 'individual-plots.json'), {}, settings).then(response => {
                if (response.status === 200) {
                    return true;
                }
                else {
                    return false;
                }
            });
        }
        return new Promise(resolve => {
            // Hack Alert! We would like to test whether a given URL is actually
            // a dask dashboard, since we will be iframe-ing it sight-unseen.
            // However, CORS policies prevent us from doing a normal fetch
            // to an arbitrary URL. We *can*, however, request an image from
            // an arbitrary location. So let's request the dask logo from the
            // bokeh server statics directory and check whether that was successful.
            //
            // If the logo ever moves or changes names, or if there is a different
            // server with an identical file path, then this will fail.
            let logoUrl = coreutils_1.URLExt.join(url, 'statics/images/dask-logo.svg');
            // Bust caching for src attr
            // https://developer.mozilla.org/en-US/docs/Web/API/XMLHttpRequest/Using_XMLHttpRequest#Bypassing_the_cache
            logoUrl += (/\?/.test(logoUrl) ? '&' : '?') + new Date().getTime();
            let img = document.createElement('img');
            img.onload = () => {
                resolve(true);
            };
            img.onerror = () => {
                resolve(false);
            };
            img.src = logoUrl;
        });
    }
    Private.testDaskDashboard = testDaskDashboard;
    function createInactivePanel() {
        const panel = document.createElement('div');
        panel.className = 'dask-DaskDashboard-inactive';
        return panel;
    }
    Private.createInactivePanel = createInactivePanel;
    /**
     * Test whether the url is a local url.
     *
     * #### Notes
     * This function returns `false` for any fully qualified url, including
     * `data:`, `file:`, and `//` protocol URLs.
     */
    function isLocal(url) {
        const { protocol } = coreutils_1.URLExt.parse(url);
        return (url.toLowerCase().indexOf(protocol) !== 0 && url.indexOf('//') !== 0);
    }
    Private.isLocal = isLocal;
})(Private || (Private = {}));
