"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const apputils_1 = require("@jupyterlab/apputils");
const React = require("react");
/**
 * A component for an HTML form that allows the user
 * to select scaling parameters.
 */
class ClusterScaling extends React.Component {
    /**
     * Construct a new ClusterScaling component.
     */
    constructor(props) {
        super(props);
        let model;
        const adaptive = !!props.initialModel.adapt;
        // If the initial model is static, enrich it
        // with placeholder values for minimum and maximum workers.
        if (!adaptive) {
            model = Object.assign(Object.assign({}, props.initialModel), { adapt: {
                    minimum: props.initialModel.workers,
                    maximum: props.initialModel.workers
                } });
        }
        else {
            model = props.initialModel;
        }
        this.state = { adaptive, model };
    }
    /**
     * When the component updates we take the opportunity to write
     * the state of the cluster to an external object so this can
     * be sent as the result of the dialog.
     */
    componentDidUpdate() {
        let model = Object.assign({}, this.state.model);
        if (!this.state.adaptive) {
            model.adapt = null;
        }
        this.props.stateEscapeHatch(model);
    }
    /**
     * React to the number of workers changing.
     */
    onScaleChanged(event) {
        this.setState({
            model: Object.assign(Object.assign({}, this.state.model), { workers: parseInt(event.target.value, 10) })
        });
    }
    /**
     * React to the user selecting the adapt checkbox.
     */
    onScalingChanged(event) {
        const value = event.target.checked;
        this.setState({
            model: this.state.model,
            adaptive: value
        });
    }
    /**
     * React to the minimum slider changing. We also update the maximum
     * so that it is alway greater than or equal to the minimum.
     */
    onMinimumChanged(event) {
        const value = parseInt(event.target.value, 10);
        const minimum = Math.max(0, value);
        const maximum = Math.max(this.state.model.adapt.maximum, minimum);
        this.setState({
            model: Object.assign(Object.assign({}, this.state.model), { adapt: {
                    maximum,
                    minimum
                } })
        });
    }
    /**
     * React to the maximum slider changing. We also update the minimum
     * so that it is always less than or equal to the maximum.
     */
    onMaximumChanged(event) {
        const value = parseInt(event.target.value, 10);
        const maximum = Math.max(0, value);
        const minimum = Math.min(this.state.model.adapt.minimum, maximum);
        this.setState({
            model: Object.assign(Object.assign({}, this.state.model), { adapt: {
                    maximum,
                    minimum
                } })
        });
    }
    /**
     * Render the component..
     */
    render() {
        const model = this.state.model;
        const adapt = model.adapt;
        const adaptive = this.state.adaptive;
        const disabledClass = 'dask-mod-disabled';
        return (React.createElement("div", null,
            React.createElement("span", { className: "dask-ScalingHeader" }, "Manual Scaling"),
            React.createElement("div", { className: "dask-ScalingSection" },
                React.createElement("div", { className: "dask-ScalingSection-item" },
                    React.createElement("span", { className: `dask-ScalingSection-label ${adaptive ? disabledClass : ''}` }, "Workers"),
                    React.createElement("input", { className: "dask-ScalingInput", disabled: adaptive, value: model.workers, type: "number", step: "1", onChange: evt => {
                            this.onScaleChanged(evt);
                        } }))),
            React.createElement("div", { className: "dask-ScalingHeader" },
                "Adaptive Scaling",
                React.createElement("input", { className: "dask-ScalingCheckbox", type: "checkbox", checked: adaptive, onChange: evt => {
                        this.onScalingChanged(evt);
                    } })),
            React.createElement("div", { className: "dask-ScalingSection" },
                React.createElement("div", { className: "dask-ScalingSection-item" },
                    React.createElement("span", { className: `dask-ScalingSection-label ${!adaptive ? disabledClass : ''}` }, "Minimum workers"),
                    React.createElement("input", { className: "dask-ScalingInput", disabled: !adaptive, type: "number", value: adapt.minimum, step: "1", onChange: evt => {
                            this.onMinimumChanged(evt);
                        } }))),
            React.createElement("div", { className: "dask-ScalingSection" },
                React.createElement("div", { className: "dask-ScalingSection-item" },
                    React.createElement("span", { className: `dask-ScalingSection-label ${!adaptive ? disabledClass : ''}` }, "Maximum workers"),
                    React.createElement("input", { className: "dask-ScalingInput", disabled: !adaptive, type: "number", value: adapt.maximum, step: "1", onChange: evt => {
                            this.onMaximumChanged(evt);
                        } })))));
    }
}
exports.ClusterScaling = ClusterScaling;
/**
 * Show a dialog for scaling a cluster model.
 *
 * @param model: the initial model.
 *
 * @returns a promse that resolves with the user-selected scalings for the
 *   cluster model. If they pressed the cancel button, it resolves with
 *   the original model.
 */
function showScalingDialog(model) {
    let updatedModel = Object.assign({}, model);
    const escapeHatch = (update) => {
        updatedModel = update;
    };
    return apputils_1.showDialog({
        title: `Scale ${model.name}`,
        body: (React.createElement(ClusterScaling, { initialModel: model, stateEscapeHatch: escapeHatch })),
        buttons: [apputils_1.Dialog.cancelButton(), apputils_1.Dialog.okButton({ label: 'SCALE' })]
    }).then(result => {
        if (result.button.accept) {
            return updatedModel;
        }
        else {
            return model;
        }
    });
}
exports.showScalingDialog = showScalingDialog;
